/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

%macro loadappcfg(prefix=EV,     /* prefix identifying properties of interested */
                  fullPropKey=,  /* the full property key value (inc. prefix) for a single property to load */
                  appName=,      /* Name of application */
                  scope=G,       /* Symbol table in which to create macro variables */
                  abendOnErr=N,  /* If no configuration props found, abend? */
                  out=__TEMP);   /* name of data set to create; default is to leave no data set */

 %global cfgURL;
 %local MNAME;
 %let MNAME=loadappcfg;
 %evcomment((&MNAME) ...loading configuration properties);
 
 %if %upcase("&out") eq "_NULL_" %then %let out=__TEMP;

 %if "&cfgURL" eq "" %then
 %do; /*need to determine configuration URL */ 

/****************************************************************************/
/* NOTE: This macro assumes the following structure in the metadata:        */
/*                                                                          */ 
/* SoftwareComponent (contains 'Web Infra Platfrm ClntAccss')               */
/* .DeployedComponents ('Registered SAS Application')                       */
/* ..SourceConnections (all of the properties as attributes                 */
/*                                                                          */
/****************************************************************************/

 %macro check_rc(rc,zeroOK=N);

  %if "&zeroOK" eq "N" %then %let operator=LE;
  %else %let operator=LT;

  if &rc &operator 0 then
  do; /* metadata function call failed; error and bail */

     put "ERROR: Call to metadata function returned bad return code. (&rc=" &rc +(-1) ")";
     put _all_;

     call symputx("EVDMERR","Y");
     stop;

  end;/* metadata function call failed; error and bail */
%mend check_rc;

 data _null_;
  length sc_uri  $200;                                     /* SoftwareComponent Object Vars   */
  length dc_uri  $200 dc_name $200;                        /* DeployedComponent Object Vars   */
  length con_uri $200 ;                                    /* SourceConnection Object Vars    */
  length port $5 hostname $50 service $200 protocol $20;   /* SourceConnection Attribute Vars */

  retain dc_uri dc_name con_uri port hostname service protocol ""; /* suppress VAR NOT INITIALIZE msg */

  sc_uri="omsobj:SoftwareComponent?@Name=:'Web Infra Platfrm ClntAccss'";
  
  do while (rc gt 0 or rc eq .);
 
    n+1;

    rc=metadata_getnasn(sc_uri,"DeployedComponents",n,dc_uri);
    %check_rc(rc);

    rc2=metadata_getattr(dc_uri,"Name",dc_name);
    %check_rc(rc2,zeroOK=Y);

    if dc_name eq "Registered SAS Application" then
    do; /* found it */

       rc3=metadata_getnasn(dc_uri,"SourceConnections",1, con_uri);
       %check_rc(rc3);

       rc_port=metadata_getattr(con_uri,"Port",port); 
       %check_rc(rc_port,zeroOK=Y);

       rc_host=metadata_getattr(con_uri,"HostName",hostname);
       %check_rc(rc_host,zeroOK=Y);

       rc_service=metadata_getattr(con_uri,"Service",service);
       %check_rc(rc_service,zeroOK=Y);

       rc_protocol=metadata_getattr(con_uri,"CommunicationProtocol",protocol);
       %check_rc(rc_protocol,zeroOK=Y);
       
       url=cats(protocol,"://",hostname,":",port,service,"/rest/applications");

       put "NOTE: URL for WIP Configuration Service extracted from SAS Metadata Server; URL=" url;
       call symputx("CFGURL",url,'G');

       leave;
    end;/* found it */

  end; /* do until */

  if missing(url) then
  do; /* no configuration URL was generated */

     put "ERROR: The URL for the WIP Configuration Services could NOT be determined based on the information found in the SAS Metadata Server";
     put _all_;

     call symputx("EVDMERR","Y");

  end;/* no configuration URL was generated */
  else call symputx("EVDMERR","N");
 run;

 %end;/*need to determine configuration URL */

 filename hdrout temp;
 filename metaProp temp;
 filename _xmlmap "&sasevinstalldir/SASJobs/metaconfig.xmlmap";

 proc http out=metaProp url="&cfgURL./&appName" http_tokenauth  method="GET" ct="application/xml" headerout=hdrout;
 run;

 %local status;
 %gethttpstatus(headerout=hdrout,returnvar=status);

 %if &syserr ne 0 or &status ne 0 %then 
 %do; /* unable to connect */
 
     %put %str(ERROR: Configuration Properties Query failed to return any properties; unable to connect to the WIP Configuration Services.(syserr= &syserr status=&status));
     
     %if "&abendOnErr" eq "Y" %then 
     %do; /* abort on error flag set */
        %put ERROR: Aborting with no further processing.;
        %abort return 253;
     %end;/* abort on error flag set */
     
 %end;/* unable to connect */


 libname   metaProp xmlv2 xmlmap=_xmlmap access=READONLY;

 data _null_;     
  cfgDataExist=exist("metaprop.setting");

  if not (cfgDataExist) then
  do; /* configuration properties not returned */
     put "ERROR: Configuration Properties Query failed to return any properties; unable to load configuration properties.";
     put "ERROR: Aborting with no further processing.";
     abort return 253;
  end;/* configuration properties not returned */
 run;


 data &out;
  length key $50 name $50 mvarname $ 32 value $200;
  set metaprop.setting end=lastrec;


 %if "&fullPropKey" ne "" %then
 %do; /*specific property requested*/ 

  if key eq "&fullPropKey.";

 %end;/*specific property requested */

 %else %if "&prefix" ne "" %then
 %do; /*prefix specified */ 

  if key eq: "&prefix.";

 %end;/*prefix specified */

  name= compress(scan(key,-1,'.'),".");

  if length(name) gt 32 then
  do; /* cfg prop name is too long */

    putlog "WARNING: Configuration Key value," name" is TOO LONG for a SAS macro variable name; this property will be ignored.";
    delete;

  end;/* cfg prop name is too long */

  mvarname=name;

  if notname(trim(mvarname)) then 
  do; /* property name is NOT a valid SAS macro var name */

    putlog "WARNING: Configuration Key value," mvarname" is NOT  a valid SAS macro variable name; property will be ignored.";
    mvarname="";
    delete;

  end;/* property name is NOT a valid SAS macro var name */
  
  %if &evdebug GT 1 %then %do; /* write values to the log */
 	putlog "DEBUG: (&MNAME) " mvarname= value=;
  %end;
  
  if trim(mvarname) = "evdebug" then do;
  	if symget('evdebug') NE '' then do;
  	   putlog "NOTE: Overriding evdebug default configuration setting " value=;
        call symputx(mvarname,"&evdebug","&scope");
     end;
  end;
  else do;
  	call symputx(mvarname,value,"&scope");
  end;
 run;

 %local nobs;
 %getnobs(data=&out,mvar=nobs);

 %if "&nobs" lt "1" and "&abendOnErr" eq "Y" %then 
 %do; /* no properties met specified conditions */

    %put ERROR: No Configuration Properties were found that met the specfied criteria.;
    %let evdmerr="Y";
    %abort return 252;

 %end;/* no properties met specified conditions */
 %else %if "&nobs" lt "1" %then 
 %do; /* no properties met specified conditions */

    %put WARNING: No Configuration Properties were found that met the specfied criteria.;

 %end;/* no properties met specified conditions */

 %else %put NOTE: &nobs Configuration Properties were read from the SAS Metadata Server.;

 %if %upcase("&out") eq "__TEMP" %then
 %do; /* delete temp data set */

    proc datasets lib=work nolist;
     delete __temp;
     run;
    quit;

 %end;/* delete temp data set */

%mend loadappcfg;
